"""This library is intended to provide Python 2.x support for the ISaAC (Ideal System and Application Circuit) expansion board for Raspberry Pi.
The ISaAC board, additional docmentation, and assitances are all available from http://www.kibacorp.com
Error handling is included in the library to provide smooth execution
Please forward any questions or bug reports to kibacorpinc@gmail.com
Several functions are provided to ease the burden on the end-user for serial-based function calls to the board.

These functions include:
    - simple command transfer(H,L,O,I)
    - analog to digital measurement by pin
    - pulse with modulation by pin 9 or 10
    - EEPROM data storage write
    - real time clock calendar time set
    - DAC out
       - Servo control
       -blink

Revision History:
2014  Jan 6     version 0.1 btk
2014 July 18    version 0.2  thk
    - initial creation of library
    - API_com function included to send raw commands
    - API_ADC function included to send simplified analog read requests
    - check_clear function included to clear serial buffer
    - servo_position function included to set servo based on min/max and 10-bit value
    - blink function added for simple blink LED by pin
    _exteneded serial timeout to accommodate Clock set time (.7 sec)
    -added Digital_in(pin)for digital in
    -added ClockRead() to read\clock once set
    -added ClockWrite(yr,mth,day,hour,minte,sec,day of week)
    -added EEPROM_read(addr)
    -added EEPROM_write(addr,data)
    -added DAC_out(data)
    -added PWM (pin, duty_cycle)
   T,Q,V,X,U not implemented 
    

"""

import time
import serial

#initialize the serial port, 19200 8N1
ser = serial.Serial(port='/dev/ttyAMA0', 
                    baudrate=19200, 
                    bytesize=serial.EIGHTBITS, 
                    parity=serial.PARITY_NONE, 
                    stopbits=serial.STOPBITS_ONE,
                    timeout=0.7)



#initialize the pins and flush the serial port
def check_clear():
    """This function is interally called to ensure the serial port is clear before commands are sent"""
    while ser.inWaiting() !=0:
        ser.readline()


def API_com(string_com):  #use this command just like a serial terminal
    """This function provides a simple capability to send commands directly to ISaAC board.
    The intention is that this is primarily used for debugging or prototyping, but it can be used for final scripts as well.

    The list of available commands is provided below:

    NOTE: All arguments passed must be as a string

    NOTE: The function returns a 2-element array, with echo from ISaAC and then A for ACK or N for NACK
    NOTE: If the function fails a 2-element array ("ERR","ERR") is returned

    ***ISAAC Board API ver8 ***
    
   *************4 character comands: I,O,H,L ?, A
    format (1 ASCII CMD)  (2 ASCII Channel) CR
    I-set digital to input -response A=ACK/N=NACK
    O-set digital to output -response A=ACK/N=NACK
    H-set digital to high level -response A=ACK/N=NACK
    L-set digital to low level -response A=ACK/N=NACK
   
   
    """    
    try:
        ser.open()
        check_clear()
        ser.write(string_com+ '\r')
        echo_info = ser.readline()
        return_acknack = ser.readline()
        echo, foo = echo_info.split("\r")
        acknack, foo = return_acknack.split("\r")
        check_clear()
        ser.close()
        return (echo, acknack)

    except:
        return ("ERR","ERR")
    
def ClockSet(yr,mth,day,hour,minute,sec,dayofweek):  
    """    ***ISAAC Board API ver8 HELP****
    CS clock yesr,month,day,hours,minutes, seconds, dayof week
    All strings: year(2) tens/units, month(2) ten/units, day (2)tens/units,
    hours (2) Military time tens/units, 
    minutes(2) tens/units, seconds (2) tens/units,day of week(1)
    0-6 (0= sunday 6 =saturday)
    """    
    try:
        ser.open()
        check_clear()
        ser.write('CS'+yr+mth+day+hour+minute+sec+dayofweek+ '\r')
        echo_info = ser.readline()
        return_acknack = ser.readline()
        echo, foo = echo_info.split("\r")
        acknack, foo = return_acknack.split("\r")
        check_clear()
        ser.close()
        return (echo, acknack)

    except:
        return ("ERR","ERR")
    
def ClockRead():
    """CR clock read returns (2) 16 character string time and date
    Must use API_com with CS clock set) first """
    try:
        ser.open()
        check_clear()
        ser.write( "CR"+ '\r')
        echo_info = ser.readline()
        return_acknack = ser.readline()
        return_date=ser.readline()
        echo, foo = echo_info.split("\r")
        time, foo = return_acknack.split("\r")
        date, foo = return_date.split("\r")
        check_clear()
        ser.close()
        return (time,date)

    except:
        return ("ERR","ERR")
        
    
def Digital_in(pin_num):
    """ ?-read and report digital pin value-  ACK 0,1 or N(NACK)
    This function accepts a 2 digit string for pin number test pin digital input value
    The function will return a 2-component array.  The first component is the echo resposne from the ISaAC board
    The second componet is a 0 or 1 for the digital value of that pin.
    For example, API_Q("01") would return ("A01",1) for a successful read
    If the reading fails or an improper input is given, the function will return the array as (ERR, 0)
    """

    try:
            ser.open()
            check_clear()
            ser.write("?" + pin_num + "\r")
            echo_info = ser.readline()
            echo, foo = echo_info.split("\r")
            in_data = ser.readline()
            check_clear()
            ser.close()
            foo, num1_in = in_data.split("A")
           
          
            return (echo, num1_in[0])
        
    except:
            return ("ERR", 0)

    else:
        
        return ("ERR", 0)
    

def API_ADC(pin_num):
    """This function accepts a 2 digit string for pin number to measure (14, 15, 16, 17)
    The function will return a 2-component array.  The first component is the echo resposne from the ISaAC board
    The second componet is a 10-bit integer value for the analog value of that pin.
    For example, API_ADC("14") would return ("A14",1023) for a successful read
    If the reading fails or an improper input is given, the function will return the array as (ERR, 0)"""
    if int(pin_num) >= 14 and int(pin_num) <=17:
        try:
            ser.open()
            check_clear()
            ser.write("A" + pin_num + "\r")
            echo_info = ser.readline()
            echo, foo = echo_info.split("\r")
            ADC_data = ser.readline()
            check_clear()
            ser.close()
            foo, num1_ADC = ADC_data.split("A")
            num2_ADC, foo, poo = num1_ADC.split("\r")
            int_ADC = int(num2_ADC)
            return (echo, int_ADC)
        
        except:
            return ("ERR", 0)

    else:
        print "Please choose pin 14, 15, 16, or 17\n"
        return ("ERR", 0)
def EEPROM_write(addr,data):  
    """    ***ISAAC Board API ver8 HELP****
    address is string '000' to '127'
    data is string  '000' to '255'
    """    
    try:
        ser.open()
        check_clear()
        ser.write('EW'+addr+data + '\r')
        echo_info = ser.readline()
        return_acknack = ser.readline()
        echo, foo = echo_info.split("\r")
        acknack, foo = return_acknack.split("\r")
        check_clear()
        ser.close()
        return (echo, acknack)

    except:
        return ("ERR","ERR")
    
def EEPROM_read(addr):
    """This function accepts a 2 digit string for pin number to measure (14, 15, 16, 17)
    The function will return a 2-component array.  The first component is the echo resposne from the ISaAC board
    The second componet is a 10-bit integer value for the analog value of that pin.
    For example, API_ADC("14") would return ("A14",1023) for a successful read
    If the reading fails or an improper input is given, the function will return the array as (ERR, 0)"""
   
    try:
            ser.open()
            check_clear()
            ser.write("ER" + addr + "\r")
            echo_info = ser.readline()
            EEPROM_data = ser.readline()
            echo, foo = echo_info.split("\r")
            foo, data = EEPROM_data.split("A")
            read, foo,poo = data.split("\r")
            check_clear()
            ser.close()
            return (read)
        
    except:
            return ("ERR", 0)

    else:
        print "Please choose pin 14, 15, 16, or 17\n"
        return ("ERR", 0)
def DAC_out(data):  
    """    ***ISAAC Board API ver8 HELP****
    
    data is string  '0000' to '1023'
    """    
    try:
        ser.open()
        check_clear()
        ser.write('D'+data + '\r')
        echo_info = ser.readline()
        return_acknack = ser.readline()
        echo, foo = echo_info.split("\r")
        acknack, foo = return_acknack.split("\r")
        check_clear()
        ser.close()
        return (echo, acknack)

    except:
        return ("ERR","ERR")
    
def PWM(pin, duty_cycle):  
    """    ***ISAAC Board API ver8 HELP****
    pin string '09' or '10'
    duty_cycle  is string  '000' to '255'
    """    
    try:
        ser.open()
        check_clear()
        ser.write('P'+pin +duty_cycle + '\r')
        echo_info = ser.readline()
        return_acknack = ser.readline()
        echo, foo = echo_info.split("\r")
        acknack, foo = return_acknack.split("\r")
        check_clear()
        ser.close()
        return (echo, acknack)

    except:
        return ("ERR","ERR")
    
def blink(pin, time_val = 0.05):
    """This function can be called for any available pin on the ISaAC board
    The pin number must be provided as a two digit string, i.e. pin 1 is "01" pin 10 is "10", etc
    The assumption is that the pin has previously been set to ouput and a simple high efficiency LED is attached to the pin
    Default blink time is set to 50 milliseconds, but may be modifed during function call
    The function will return integer 1 if successul, if any error occurs a integer 0 is returned"""
    
    try:
        API_com("H" + pin)
        time.sleep(time_val)
        API_com("L" + pin)
        time.sleep(time_val)
        return 1
    
    except:
        return 0

#define set servo position function
def servo_position(servo_value, servo_pin, min_val=1000, max_val=2000):
    
    if servo_value > 1023 or servo_value < 0:
        print "Please check that servo position is set between min (" + min_val + ") and max (" + max_val + ")"
        return ("ERR","ERR")

    try:
        ser.open()
        check_clear()
        servo_tick_per_ADC = float(max_val - min_val) / 1023
        servo_pos = int(min_val + servo_value // servo_tick_per_ADC )
        command_str = ("Z" + servo_pin + "00050000000" + str(servo_pos) + "\r")
        ser.write(command_str)
        echo = ser.readline()
        acknack = ser.readline()
        ser.close()
        return (echo, acknack)

    except:
        return ("ERR","ERR")

